classdef KinematicCarGraphic < handle
    
    properties (SetAccess=private, GetAccess=public)
        l=2;    % wheelbase
        w=2/2;    % axle length
        ww=2/3; % wheel diameter
        h;      % line handles
    end
    properties (SetAccess=private, GetAccess=private, Hidden = true)
        vs;     % graphic vertices
        vr;     % graphic vertices
    end
    methods
        function obj = KinematicCarGraphic(ax,l,cstr)
            obj.l=l;
            w=l/2;
            ww=l/3;
            obj.w = w;
            obj.ww = ww;
            
            % line vertices as matrix for N lines (each line defined by pair [xA xB;yA yB])
            % v = [xA1 xB1 xA2 xB2 ... xAN xBN;
            %      yA1 xB1 yA2 yB2 ... yAN yBN];

            % rear axle
            obj.vr = [0     0   -ww/2   ww/2    -ww/2   ww/2;
                     -w/2   w/2  w/2    w/2     -w/2   -w/2];

            % longitudinal axis
            va = [ 0   l;
                   0   0];
              
            % add front axle assemb
            vf = obj.frontAxle(0);

            % put car together
            obj.vs = [obj.vr va];
            v = [obj.vs vf];

            % draw the car
            axes(ax);
            obj.h=line(reshape(v(1,:),2,[]),reshape(v(2,:),2,[]));
            set(obj.h,'Color',cstr,'LineWidth',2)
        end
        
        function v = frontAxle(obj,phi)
            v = obj.transform(obj.vr,[obj.l;0],phi);
        end
        
        function obj = update(obj,y,theta,phi)
            vf = obj.frontAxle(phi);
            v = obj.transform([obj.vs vf],y,theta);
            for i=1:length(obj.h)
                set(obj.h(i),'XData',reshape(v(1,2*i-1:2*i),2,[]),'YData',reshape(v(2,2*i-1:2*i),2,[])); 
            end
        end
        function vt = transform(obj,v,y,theta)
            R = [cos(theta) -sin(theta);sin(theta) cos(theta)];
            vt = repmat(y,1,size(v,2))+R*v;
        end
        function timer_update_fcn (obj, tmr, ~, y1, y2, theta, phi)
            idx = tmr.UserData + 1;
            tmr.UserData = idx;
            obj.update([y1(idx);y2(idx)],theta(idx),phi(idx));
        end
        function timer_start_fcn(obj, tmr,~)
            tmr.UserData = 1;
        end
        function timer_stop_fcn (obj, tmr, ~)
            %delete(tmr);
        end
    end
end